import QtQuick 2.0
import Sailfish.Silica 1.0
import "private"

Column {
    id: dialer

    property alias voiceMailIconSource: voiceMailIcon.source
    property variant vanityDialNumbers: ["", "abc", "def", "ghi", "jkl", "mno", "pqrs", "tuv", "wxyz", "+", "", ""]
    property bool vanityDialNumbersVisible: true
    property bool symbolsVisible: true
    property color textColor: Theme.primaryColor
    property color pressedTextColor: Theme.highlightColor
    property alias pressedButtonColor: pressedButtonBackground.color

    property string _numbers: "0123456789"
    property QtObject _feedbackEffect

    signal pressed(string number)
    signal released(string number)
    signal canceled(string number)
    signal clicked(string number)

    signal voiceMailCalled

    function _buttonPressed(item, number) {
        if (_feedbackEffect) {
            _feedbackEffect.play()
        }
        _centerPressedButtonBackgroundOnItem(item)
        pressedButtonBackground.opacity = 1
        pressedButtonBackground.visible = true
        dialer.pressed(number)
    }

    function _buttonReleased(number) {
        pressedButtonBackground.visible = false
        dialer.released(number)
    }

    function _buttonCanceled(number) {
        pressedButtonBackground.visible = false
        dialer.canceled(number)
    }

    function _buttonClicked(number) {
        dialer.clicked(number)
    }

    function _buttonEntered() {
        pressedButtonBackground.opacity = 1
    }

    function _buttonExited() {
        pressedButtonBackground.opacity = 0
    }

    function _centerPressedButtonBackgroundOnItem(item) {
        pressedButtonBackground.x = 0
        pressedButtonBackground.y = 0
        var itemCenter = pressedButtonBackground.mapFromItem(item, item.width/2, item.height/2)
        pressedButtonBackground.x = itemCenter.x - pressedButtonBackground.width/2
        pressedButtonBackground.y = itemCenter.y - pressedButtonBackground.height/2
    }

    anchors {
        left: parent.left
        leftMargin: Theme.paddingLarge * 2
        right: parent.right
        rightMargin: Theme.paddingLarge * 2
    }

    Component.onCompleted: {
        // Avoid hard dependency to feedback
        _feedbackEffect = Qt.createQmlObject("import QtQuick 2.0; import QtFeedback 5.0; ThemeEffect { effect: ThemeEffect.PressWeak }",
                           dialer, 'ThemeEffect');
    }

    Item {
        // Place button background here and not in the root Column so it can be repositioned.
        Rectangle {
            id: pressedButtonBackground
            width: (dialer.width / 3) - (2 * Theme.paddingSmall)
            height: width - Theme.paddingSmall  // avoid touching vanity characters
            visible: false
            radius: 4

            // same as BackgroundItem
            color: Theme.rgba(Theme.highlightBackgroundColor, Theme.highlightBackgroundOpacity)
        }
    }

    Row {
        KeypadButton {
            key: Qt.Key_1
            text: _numbers.charAt(1)
            secondaryText: vanityDialNumbers[0]
            onPressAndHold: dialer.voiceMailCalled()
            Image {
                id: voiceMailIcon
                anchors {
                    bottom: parent.bottom
                    horizontalCenter: parent.horizontalCenter
                }
            }
        }
        KeypadButton {
            key: Qt.Key_2
            text: _numbers.charAt(2)
            secondaryText: vanityDialNumbers[1]
        }
        KeypadButton {
            key: Qt.Key_3
            text: _numbers.charAt(3)
            secondaryText: vanityDialNumbers[2]
        }
    }
    Row {
        Repeater {
            model: 3
            KeypadButton {
                key: Qt.Key_4 + index
                text: _numbers.charAt(4 + index)
                secondaryText: vanityDialNumbers[3 + index]
            }
        }
    }
    Row {
        Repeater {
            model: 3
            KeypadButton {
                key: Qt.Key_7 + index
                text: _numbers.charAt(7 + index)
                secondaryText: vanityDialNumbers[6 + index]
            }
        }
    }
    Row {
        Item {
            width: asteriskButton.width
            height: asteriskButton.height

            KeypadButton {
                id: asteriskButton
                visible: symbolsVisible
                key: Qt.Key_Asterisk
                text: "*"
                secondaryText: vanityDialNumbers[9]
            }
        }
        KeypadButton {
            key: Qt.Key_0
            text: "0"
            secondaryText: vanityDialNumbers[10]
        }        
        Item {
            width: hashButton.width
            height: hashButton.height

            KeypadButton {
                id: hashButton
                visible: symbolsVisible
                text: "#"
                key: Qt.Key_NumberSign
                secondaryText: vanityDialNumbers[11]
            }
        }
    }
}
