import QtQuick 2.0

Item {
    id: splitImage
    property string source

    property bool asynchronous: false

    implicitWidth: topImage.implicitWidth
    implicitHeight: topImage.implicitHeight+bottomImage.implicitHeight
    property size sourceSize: Qt.size(0, 0)
    property size _halfSourceSize: Qt.size(sourceSize.width, sourceSize.height/2)

    property int status: Image.Null

    function updateStatus()
    {
        if (topImage.status == Image.Null || bottomImage.status == Image.Null)
            status = Image.Null
        else if (topImage.status == Image.Loading || bottomImage.status == Image.Loading)
            status = Image.Loading
        else if (topImage.status == Image.Error || bottomImage.status == Image.Error)
            status = Image.Error
        else
            status = Image.Ready
    }

    Image {
        id: topImage
        source: splitImage.source ? "image://split/"+splitImage.source+"?top" : ""
        asynchronous: splitImage.asynchronous

        sourceSize: _halfSourceSize

        width: parent.width
        height: parent.height/2

        onStatusChanged: updateStatus()
    }

    Image {
        id: bottomImage
        source: splitImage.source ? "image://split/"+splitImage.source+"?bottom" : ""
        asynchronous: splitImage.asynchronous

        sourceSize: _halfSourceSize

        width: parent.width
        height: parent.height/2
        anchors {
            top: topImage.bottom
        }

        onStatusChanged: updateStatus()
    }
}
