import QtQuick 2.1
import QtQuick.Window 2.1
import Sailfish.Silica 1.0
import Sailfish.Silica.private 1.0

Item {
    id: wallpaper

    property alias source: wallpaperTextureImage.source

    property real horizontalOffset: 0
    property real verticalOffset: -Screen.height / 3

    Item {
        id: glassTextureItem
        visible: false
        width: glassTextureImage.width
        height: glassTextureImage.height
        Image {
            id: glassTextureImage
            opacity: 0.1
            source: "image://theme/graphic-shader-texture"
            Behavior on opacity { FadeAnimation { duration: 200 } }
        }
    }

    Image {
        id: wallpaperTextureImage
        visible: false
        source: Theme.backgroundImage
        onSourceChanged: {
            // Workaround -- seems to be necessary for the ShaderEffect to update the texture
            wallpaperEffect.wallpaperTexture = null
            wallpaperEffect.wallpaperTexture = wallpaperTextureImage
        }
    }

    ShaderEffect {
        id: wallpaperEffect
        anchors.fill: parent

        visible: wallpaperTextureImage.source != ""

        // offset normalized to screen size
        property size offset: Qt.size(wallpaper.horizontalOffset / Screen.width, wallpaper.verticalOffset / Screen.height)

        // ratio of screen size vs home wallpaper size
        property real ratio: (Screen.height/Screen.width)/(wallpaperTextureImage.implicitHeight/wallpaperTextureImage.implicitWidth)
        property size sizeRatio: Qt.size(1., ratio)

        // glass texture size
        property size glassTextureSizeInv: Qt.size(1.0/(Theme.pixelRatio*glassTextureImage.sourceSize.width), -1.0/(Theme.pixelRatio*glassTextureImage.sourceSize.height))

        property Image wallpaperTexture: wallpaperTextureImage
        property variant glassTexture: ShaderEffectSource {
            hideSource: true
            sourceItem: glassTextureItem
            wrapMode: ShaderEffectSource.Repeat
        }

        // Enable blending in compositor (for events view etc..)
        blending: !Config.wayland

        vertexShader: "
           uniform highp mat4 qt_Matrix;
           uniform highp vec2 offset;
           uniform highp vec2 sizeRatio;
           attribute highp vec4 qt_Vertex;
           attribute highp vec2 qt_MultiTexCoord0;
           varying highp vec2 qt_TexCoord0;
           void main() {
              qt_TexCoord0 = (qt_MultiTexCoord0 - offset) * sizeRatio;
              gl_Position = qt_Matrix * qt_Vertex;
           }
        "

        fragmentShader: "
           uniform sampler2D wallpaperTexture;
           uniform sampler2D glassTexture;
           uniform highp vec2 glassTextureSizeInv;
           uniform lowp float qt_Opacity;
           varying highp vec2 qt_TexCoord0;
           void main() {
              lowp vec4 wp = texture2D(wallpaperTexture, qt_TexCoord0);
              lowp vec4 tx = texture2D(glassTexture, gl_FragCoord.xy * glassTextureSizeInv);
              gl_FragColor = vec4(0.4*wp.rgb + tx.rgb, 1.0)" + (blending ? "*qt_Opacity" : "") + ";
           }
        "
    }
}
